package org.irishrobotics.model;

import java.sql.*;
import java.util.Iterator;
import java.util.Vector;

public class Model
	{
	public static Vector parsePosMsg(byte[] b)
		{
		Vector v = new Vector();
		int c,len,sn = 0,num_positions;
		byte cs = 0;
		
		//Checksum the message.
		for(c = 0;c < b.length - 1;c++)
			cs ^= (int)b[c] & 0xFF;
		if(cs != b[b.length -1])
			return v;
			
		//Extract the msg length and set the number of position reports it contains.
		len = (int)b[MSGConstants.LEN_HI_OFFSET] & 0xFF;
		len <<= 8;
		len |= (int)b[MSGConstants.LEN_LO_OFFSET] & 0xFF;
		
		//Calc the number of positions the message contains.
		num_positions = (len - MSGConstants.MSG_OVERHEAD) / GPSConstants.GPS_POS_REP_LEN;
		if(((len - MSGConstants.MSG_OVERHEAD) % GPSConstants.GPS_POS_REP_LEN) != 0)
			{
			System.out.println("message length is wrong");
			return v;
			}
		
		//Extract the serial number.
		for(c = 0,sn = 0;c < 4;c++)
			{
			sn <<= 8;
			sn |= (int)b[MSGConstants.SER_NUM_START + c] & 0xFF;
			}
		
		//Populate the vector with position records.
		for(c = 0;c < num_positions;c++)
			{
			PositionRecord pr = PositionRecord.parse(sn,b,MSGConstants.DATA_START_OFFSET + (c * GPSConstants.GPS_POS_REP_LEN));
			v.add(pr);
			}
		return v;
		}
	
	public static void savePosRecList(Connection con,Vector v)throws SQLException
		{
		PreparedStatement i_pstmt = null;
		PreparedStatement u_pstmt = null;
		PositionRecord pr = null;
		Iterator it;
		final String SQL_INSERT = "insert into incoming_msgs (ser_num,lat,lng,alt,speed,dir,ts,ign,temperature) values(?,?,?,?,?,?,?,?,?)";
		final String SQL_UPDATE = "update cur_positions set lat = ?, lng = ?, alt = ?, speed = ?, dir = ?, ts = ?, ign = ?, temperature = ? where ser_num = ?";
		
		if(v.isEmpty())
			return;
			
		try
			{
			i_pstmt = con.prepareStatement(SQL_INSERT);
			u_pstmt = con.prepareStatement(SQL_UPDATE);
			con.setAutoCommit(false);
			
			//Insert all positions into incoming_msgs
			it = v.iterator();
			while(it.hasNext())
				{
				pr = (PositionRecord)it.next();
				i_pstmt.setInt(1,pr.getSerNum());
				i_pstmt.setFloat(2,pr.getLat());
				i_pstmt.setFloat(3,pr.getLng());
				i_pstmt.setFloat(4,pr.getAlt());
				i_pstmt.setInt(5,pr.getSpeed());
				i_pstmt.setInt(6,pr.getDir());
				i_pstmt.setTimestamp(7,pr.getTimestamp());
				i_pstmt.setInt(8,pr.getIgn());
				i_pstmt.setInt(9,pr.getTemp());
				i_pstmt.executeUpdate();
				}
				
			//Update the units current position
			u_pstmt.setInt(9,pr.getSerNum());
			u_pstmt.setFloat(1,pr.getLat());
			u_pstmt.setFloat(2,pr.getLng());
			u_pstmt.setFloat(3,pr.getAlt());
			u_pstmt.setInt(4,pr.getSpeed());
			u_pstmt.setInt(5,pr.getDir());
			u_pstmt.setTimestamp(6,pr.getTimestamp());
			u_pstmt.setInt(7,pr.getIgn());
			u_pstmt.setInt(8,pr.getTemp());
			u_pstmt.executeUpdate();
			
			con.commit();
			}
		catch(SQLException e){if(con != null) con.rollback(); throw new SQLException("savePosRecList failed");}
		finally
			{
			if(i_pstmt != null)
				i_pstmt.close();
			if(u_pstmt != null)
				u_pstmt.close();
			}
		}
	
	public static Vector loadPosRecList(Connection con,int sn,Timestamp start,Timestamp end)throws SQLException
		{
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		Vector v = new Vector();
		final String SQL = "select * from incoming_msgs where ser_num = ? and ts > ? and ts < ?";
        
		try
			{
			con.setAutoCommit(false);
            
			pstmt = con.prepareStatement(SQL);
			pstmt.setInt(1,sn);
			pstmt.setTimestamp(2,start);
			pstmt.setTimestamp(3,end);
			rs = pstmt.executeQuery();
			            
            while(rs.next())
				v.add(PositionRecord.load(rs));
			}
		catch(SQLException e){if(con != null) con.rollback(); throw new SQLException("savePosRecList failed");}
		finally
			{
			if(pstmt != null)
				pstmt.close();
			if(rs != null)
				rs.close();
			return v;
			}
		}
		
	public static PositionRecord loadCurPosition(Connection con,int sn)throws SQLException
		{
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		Vector v = new Vector();
		PositionRecord pr = null;
		final String SQL = "select * from cur_positions where ser_num = ?";
		
		try
			{
			con.setAutoCommit(false);
			pstmt = con.prepareStatement(SQL);
			pstmt.setInt(1,sn);
			rs = pstmt.executeQuery();
			if(rs.next())
				pr = PositionRecord.load(rs);
			}
		catch(SQLException e){if(con != null) con.rollback(); throw new SQLException("loadCurPosition failed");}
		finally
			{
			if(pstmt != null)
				pstmt.close();
			if(rs != null)
				rs.close();
			return pr;
			}
		}
	}