package org.irishrobotics.model;

import java.text.DateFormat;
import java.util.Calendar;
import java.io.*;
import java.sql.*;

public class PositionRecord
	{
	private int ser_num;
	private float lat;
	private float lng;
	private float alt;
	private int speed;
	private int dir;
	private Timestamp ts;
	private int ign;
	private int temp;
	
	private PositionRecord(){ts = new Timestamp(0);}
	
	public static PositionRecord load(ResultSet rs) throws SQLException
        {
        int iv;
        String sv;
        PositionRecord pr = new PositionRecord();
        
        pr.setSerNum(rs.getInt("ser_num"));
        pr.setLat(rs.getFloat("lat"));
        pr.setLng(rs.getFloat("lng"));
        pr.setAlt(rs.getFloat("alt"));
        pr.setSpeed(rs.getInt("speed"));
        pr.setDir(rs.getInt("dir"));
        pr.setTimestamp(rs.getTimestamp("ts").getTime());
        pr.setDir(rs.getInt("ign"));
        pr.setTemp(rs.getInt("temperature"));
        
        return pr;
	    }
    
    public static PositionRecord parse(int ser_num,byte[] b,int idx)
    	{
    	Calendar cal = Calendar.getInstance();
		int c,tmp = 0;
		PositionRecord pr = new PositionRecord();
		
		pr.setSerNum(ser_num);
		
		//Extract lat
		for(c = 0,tmp = 0;c < 4;c++)
			tmp |= ((int)b[idx++] & 0xFF) << (8 * c);
		pr.setLat(Float.intBitsToFloat(tmp));
		
		//Extract lng
		for(c = 0,tmp = 0;c < 4;c++)
			tmp |= ((int)b[idx++] & 0xFF) << (8 * c);
		pr.setLng(Float.intBitsToFloat(tmp));
		
		//Extract alt
		for(c = 0,tmp = 0;c < 4;c++)
			tmp |= ((int)b[idx++] & 0xFF) << (8 * c);
		pr.setAlt(Float.intBitsToFloat(tmp));
		
		//Extract velocity
		tmp = (int)b[idx++] & 0xFF;
		tmp <<= 8;
		tmp |= (int)b[idx++] & 0xFF;
		pr.setSpeed((tmp & GPSConstants.GSP_PKD_VEL_V_MSK) >> GPSConstants.GPS_PKD_VEL_SHIFT);
		
		//Extract the direction
		pr.setDir(tmp & GPSConstants.GPS_PKD_VEL_D_MSK);
		
		/*Extract the position timestamp*/
		int year = 2000 + ((int)b[idx++] & 0xFF);
		for(c = 0,tmp = 0;c < 4;c++,idx++)
			{
			tmp <<= 8;
			tmp |= (int)b[idx] & 0xFF;
			}
		
		tmp &= 0xFFFFFFC0;
		tmp >>= 6;
		
		int sec = tmp & 0x3f;
		tmp >>= 6;
		int min = tmp & 0x3F;
		tmp >>= 6;
		int hrs = tmp & 0x1F;
		tmp >>= 5;
		int day = tmp & 0x1F;
		tmp >>= 5;
		int month = tmp & 0x0F;
	
		cal.set(year,month - 1,day,hrs,min,sec);
		pr.setTimestamp(cal.getTimeInMillis());
		
		/*Get the ignition status*/
		pr.setIgn((((b[idx++] & MSGConstants.IGN_FLG_MSK) == 0) ? 0 : 1));
		pr.setTemp((int)b[idx]);
		
		return pr;
    	}
    	
    public void setSerNum(int ser_num){this.ser_num = ser_num;}
    public int getSerNum(){return ser_num;}
    public void setLat(float lat){this.lat = lat;}
    public float getLat(){return lat;}
    public void setLng(float lng){this.lng = lng;}
    public float getLng(){return lng;}
    public void setAlt(float alt){this.alt = alt;}
    public float getAlt(){return alt;}
    public void setSpeed(int speed){this.speed = speed;}
    public int getSpeed(){return speed;}
    public void setDir(int dir){this.dir = dir;}
    public int getDir(){return dir;}
    public void setTimestamp(Timestamp ts){this.ts.setTime(ts.getTime());}
    public void setTimestamp(long ms){ts.setTime(ms);}
    public Timestamp getTimestamp(){return ts;}
    public void setIgn(int ign){this.ign = ign;}
    public int getIgn(){return ign;}
    public void setTemp(int temp){this.temp = temp;}
    public int getTemp(){return temp;}
    
    public void display()
		{
		System.out.println("Serial number: " + ser_num);
		System.out.println("Date         : " + DateFormat.getInstance().format(ts));
		System.out.println("Position     : " + getLat() + ", " + getLng());
		System.out.println("Speed        : " + getSpeed() + ", Dir: " + getDir());
		System.out.println("Ignition     : " + ((getIgn() == 1) ? "ON" : "OFF"));
		System.out.println("Temperature  : " + getTemp());
		}
	}

